<?php 

Loader::helper('text');

//Loader::helper('stats',$packageHandle); 
$packageHandle='tony_stats';
$statsPkg = Package::getByHandle($packageHandle); 
require_once( $statsPkg->getPackagePath().'/helpers/stats.php' ); 

class ParticleTrafficPanel { 

	protected $title=''; //required
	
	protected $handle='';
	
	protected $displayed=1;
	protected $enabled=1;
	
	public $panel_id=0;
	
	
	public function getTitle(){
		if(strlen($this->title)) return t($this->title); 
		elseif($this->handle){
			return t(TextHelper::unhandle($this->handle)); 
		}else return t('No Panel Title or Handle Defined');
	}
	
	public function view(){
		//overrided contents optional 
	}
	
	public function render(){
		
		$c = Page::getCurrentPage();
		$statsDisplay=ParticleTrafficDisplay::$statsDisplayInstance; 
		$this->statsDisplay=$statsDisplay;
		
		$this->view();
		
		extract($this->sets);
		
		$viewPath = $this->path.'view.php';
		if( file_exists($viewPath) ) 
			include($viewPath); 
			
	}
	
	public function getPanelID(){ return $this->panel_id; }
	public function setPanelID($v){ $this->panel_id=intval($v); }
	
	public function getHandle(){ return $this->handle; }
	public function setHandle($v){ $this->handle=$v; }
	public function getPath(){ return $this->path; }
	public function setPath($v){ $this->path=$v; }
	
	public function isDisplayed(){ return $this->displayed; }
	public function setDisplayed($v){ $this->displayed=intval($v); }
	public function isEnabled(){ return $this->enabled; }
	
	public function printContents(){
		echo t('Your panel controller must override the printContents method.');  
	}
	
	
	
	private $sets = array();
	public function set($key, $val) { 
		$this->sets[$key] = $val;
	} 
	
	public function get($key = null) {  
		if (isset($this->sets[$key])) {
			return $this->sets[$key];
		}
		
		return null;
	}	
	
	public static $errorMsg='';
	static function getByHandle($handle){
		
		$db = Loader::db();
		$sql="SELECT * FROM TonyStatsPanels WHERE handle=?";
		$row = $db->getRow( $sql, array($handle) ); 
		if(!$row){
			self::$errorMsg = t('No panel was found with that handle.');
			return false; 
		}
		$controllerClassName=TextHelper::camelcase($row['handle']).'ParticleTrafficPanel';  
		if( !class_exists($controllerClassName) ){  
			$controllerPath = $row['path'].'controller.php';
			if( !file_exists($controllerPath) ){
				self::$errorMsg=t('Panel controller file not found.');
				return false; 	
			}		
			require_once($controllerPath);
			if( !class_exists($controllerClassName) ){
				self::$errorMsg=t('Panel controller class not found.');
				return false; 	
			}		
		}
		
		$panel = new $controllerClassName();
		$panel->setPanelID($row['panel_id']);
		$panel->setHandle($row['handle']);
		$panel->setPath($row['path']);
		$panel->setDisplayed($row['displayed']);
		
		return $panel; 
	}
	

	static function getPanels($col=NULL,$onlyDisplayed=0){
		
		$whereClauses=array();
		if($col) $whereClauses[]=' col="'.$col.'" '; 
		if($onlyDisplayed) $whereClauses[]= " displayed=1 "; 
	
		if( count($whereClauses) ) $whereSQL = ' WHERE '.join(' AND ',$whereClauses);
		
		$db = Loader::db();
		$sql="SELECT * FROM TonyStatsPanels ".$whereSQL." ORDER BY position, col";
		$rows = $db->getAll( $sql ); 
		$panels = array();
		foreach($rows as $row){ 
		
			//is the controller there?
			$controllerPath = $row['path'].'controller.php';
			if( !file_exists($controllerPath) ) continue; 
			
			//is the view there?
			$viewPath = $row['path'].'view.php';
			if( !file_exists($viewPath) ) continue; 
			
			//has the class been loaded?  if not, load it. 
			$controllerClassName=TextHelper::camelcase($row['handle']).'ParticleTrafficPanel'; 
			if( !class_exists($controllerClassName) ){  
				require_once($controllerPath);
				if( !class_exists($controllerClassName) ) continue; 
			}
			
			$panel = new $controllerClassName();
			$panel->setPanelID($row['panel_id']);
			$panel->setHandle($row['handle']);
			$panel->setPath($row['path']);
			$panel->setDisplayed($row['displayed']); 
			$panels[]= $panel;
		} 
		return $panels; 
	}
	

	//$path is an absolute path to your stats panel 
	static function registerPanel($path){ 
	
		$defaultPositions=array(); 
		$defaultPositions['popularity']=array('col'=>'A','position'=>1);
		$defaultPositions['user_visits']=array('col'=>'A','position'=>2);  
		$defaultPositions['users_systems']=array('col'=>'A','position'=>3);
		
		$defaultPositions['summary']=array('col'=>'B','position'=>1);
		$defaultPositions['pages']=array('col'=>'B','position'=>2);
		$defaultPositions['site_discovery']=array('col'=>'B','position'=>3);
		
		try{ 
			
			if( substr($path,-1)!='/' ) $path.='/';  
			
			$db = Loader::db(); 
			$result = array( 'success'=>0 );
			
			//test that the folder is valid
			if( !is_dir($path) ){
				$result['msg']=t('Register panel error, invalid directory:'.$path);
				return $result;
			}
			
			//test that the controller is present
			$controllerPath = $path.'controller.php';
			if( !file_exists( $controllerPath ) ){
				$result['msg']=t('Register panel error, controller file not found:'.$controllerPath);
				return $result;
			}
			
			//test that the view is present
			$viewPath = $path.'view.php';
			if( !file_exists( $viewPath ) ){
				$result['msg']=t('Register panel error, view not found:'.$viewPath);
				return $result;
			}
			
			//try to load the controller file 
			try{ 
				require_once($controllerPath); 
			}catch(Exception $e){
				$result['msg'] = t('Register panel, controller error: "').$e->getMessage().'" in '.$controllerPath;
				return $result;
			}
			
			//test that the controller class loaded correctly 
			$handle=self::getHandleFromPath($path);   
			$controllerClassName=TextHelper::camelcase($handle).'ParticleTrafficPanel'; 
			if( !class_exists($controllerClassName) ){ 
				$result['msg']=t('Register panel error, controller class '.$controllerClassName.t(' was not found in ').$controllerPath);
				return $result;
			}
			
			$panelObj = new $controllerClassName; 
			
			$title=$panelObj->getTitle();   
			
			$sql="SELECT count(*) FROM TonyStatsPanels WHERE handle= ?";
			$existingPanelCount = $db->getOne( $sql, array($handle) ); 
			
			if( $existingPanelCount ){ 
				$vals=array( $title, $path, $handle);
				$sql="UPDATE TonyStatsPanels SET title= ?, path= ? WHERE handle=?";
				$queryResult = $db->query($sql,$vals); 
			}else{
				$col = (isset($defaultPositions[$handle])) ? $defaultPositions[$handle]['col'] : 'A';
				$position = (isset($defaultPositions[$handle])) ? intval($defaultPositions[$handle]['position']) : 999;
				$vals=array($title,$handle,$path,$col,$position);
				$sql="INSERT INTO TonyStatsPanels SET title= ?, handle= ?, path= ?, col=?, position=?";  
				$queryResult = $db->query($sql,$vals); 
			}
			
			$result['success']=1;
		
		}catch(Exception $e){
			$result['msg'] = t('Register panel, unknown error: ').$e->getMessage();
			return $result;
		}	
		
		return $result;
		
	}
	
	
	static function registerDefaultPanels(){
		$packageHandle='tony_stats';
		$statsPkg = Package::getByHandle($packageHandle);   
		$dirPath = $statsPkg->getPackagePath().'/panels/';
		
		$response=array();
		$errors = array();
		$files = scandir($dirPath); 
		foreach($files as $file){
			if($file=='.' || $file=='..') continue;
			$filePath = $dirPath.$file; 
			if( is_dir($filePath) ){
				$result = self::registerPanel( $filePath ); 
				if(!$result['success']) $errors[]=$result['msg']; 
			}
		}
		$response['errors']=$errors;
		return $response;
	}
	
	
	//determine the panel handle from folder path
	static function getHandleFromPath($path){
		$dirParts=explode('/',str_replace('\\','/',$path));
		for($i=count($dirParts);$i>=0;$i--){ 
			$dirPart = $dirParts[$i];  
			if( strlen($dirPart) ) return $dirPart;  
		}
	}

}

?>